#

#{{{ setup

#   snr - measure SNR on an FMRI data set
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   Part of FSL - FMRIB's Software Library
#   http://www.fmrib.ox.ac.uk/fsl
#   fsl@fmrib.ox.ac.uk
#   
#   Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
#   Imaging of the Brain), Department of Clinical Neurology, Oxford
#   University, Oxford, UK
#   
#   
#   LICENCE
#   
#   FMRIB Software Library, Release 3.2beta (c) 2004, The University of
#   Oxford (the "Software")
#   
#   The Software remains the property of the University of Oxford ("the
#   University").
#   
#   The Software is distributed "AS IS" under this Licence solely for
#   non-commercial use in the hope that it will be useful, but in order
#   that the University as a charitable foundation protects its assets for
#   the benefit of its educational and research purposes, the University
#   makes clear that no condition is made or to be implied, nor is any
#   warranty given or to be implied, as to the accuracy of the Software,
#   or that it will be suitable for any particular purpose or for use
#   under any specific conditions. Furthermore, the University disclaims
#   all responsibility for the use which is made of the Software. It
#   further disclaims any liability for the outcomes arising from using
#   the Software.
#   
#   The Licensee agrees to indemnify the University and hold the
#   University harmless from and against any and all claims, damages and
#   liabilities asserted by third parties (including claims for
#   negligence) which arise directly or indirectly from the use of the
#   Software or the sale of any products based on the Software.
#   
#   No part of the Software may be reproduced, modified, transmitted or
#   transferred in any form or by any means, electronic or mechanical,
#   without the express permission of the University. The permission of
#   the University is not required if the said reproduction, modification,
#   transmission or transference is done without financial return, the
#   conditions of this Licence are imposed upon the receiver of the
#   product, and all original and amended source code is included in any
#   transmitted product. You may be held legally responsible for any
#   copyright infringement that is caused or encouraged by your failure to
#   abide by these terms and conditions.
#   
#   You are not permitted under this Licence to use this Software
#   commercially. Use for which any financial return is received shall be
#   defined as commercial use, and includes (1) integration of all or part
#   of the source code or the Software into a product for sale or license
#   by or on behalf of Licensee to third parties or (2) use of the
#   Software or any derivative of it for research with the final aim of
#   developing software products for sale or license to a third party or
#   (3) use of the Software or any derivative of it for research with the
#   final aim of developing non-software products for sale or license to a
#   third party, or (4) use of the Software to provide any service to an
#   external organisation for which payment is received. If you are
#   interested in using the Software commercially, please contact Isis
#   Innovation Limited ("Isis"), the technology transfer company of the
#   University, to negotiate a licence. Contact details are:
#   innovation@isis.ox.ac.uk quoting reference DE/1112.

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

#}}}

set Norm   [ YesNoWidget "Carry out intensity normalization?" "Yes" "No" ]
set IgnoreEdge [ YesNoWidget "Ignore noise at volume edges? (You probably don't want to ignore the edge-related noise if you are interested in comparing motion correction methods)" "Yes" "No" ]

#{{{ get group and create new group

MxGetCurrentGroup OrigGroup

MxGetPagesFromGroup $OrigGroup Pages

# prepare list of volumes for new group
set NewList ""

set i 1
foreach Page $Pages {
    if { $i > 2 } {
	lappend NewList $Page
    }
    incr i 1
}

MxGroupVolumes $NewList "SubGroup" Group

#}}}
#{{{ do norm if required

if { $Norm } {
    MxIntensityNormalization $Group R 10000 10
}

#}}}
#{{{ create stats images

MxWithinGrpStatistics $Group M GroupMean
name_page $GroupMean "Mean"

MxDupPage $GroupMean Mask
name_page $Mask "Mask"

MxDupPage $GroupMean Div
name_page $Div "StdDev / Mean"

MxWithinGrpStatistics $Group S GroupStdDev
name_page $GroupStdDev "StdDev"

#}}}
#{{{ find image dimensions

MxGetImageProperties $GroupMean pl
set ww [ expr [ keylget pl Width  ] - 4 ]
set hh [ expr [ keylget pl Height ] - 4 ]
set ss [ keylget pl Slices ]

#}}}
#{{{ draw box in GroupMean

# need the MxSetCurrentPage commands because MxDrawBox doesn't work properly
MxSetCurrentPage $GroupMean
MxDrawBox $GroupMean   2 2 0 $ww $hh $ss GroupMeanBox

MxQuietStatistics $GroupMeanBox { Maximum Minimum } false Stats
set Min [ keylget Stats Minimum ]
set Max [ keylget Stats Maximum ]
set Thresh [ expr ( ( $Max - $Min ) / 10.0 ) + $Min ]
puts "Threshold = $Thresh"

#}}}
#{{{ create mask image by thresholding and eroding GroupMean

MxThreshold $Mask $Thresh B1

if { $IgnoreEdge } {
    if { $ss < 8 } {
	MxSetImageOpMode $Mask 2DAll
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/circle.ker
    } else {
	MxSetImageOpMode $Mask 3D
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/sphere333.ker
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/sphere333.ker
    }
} else {
    if { $ss < 8 } {
	MxSetImageOpMode $Mask 2DAll
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/circle.ker
    } else {
	MxSetImageOpMode $Mask 3D
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/sphere333.ker
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/sphere333.ker
    }
}

MxSetImageOpMode $Mask 3D
MxThreshold $Mask 0.01 B1

#}}}
#{{{ create masked div and find final stats

MxExpr $Div = " 10000 * $Mask * ( $GroupStdDev / $GroupMean ) "

MxSetCurrentPage $Div
MxDrawBox $Div 2 2 0 $ww $hh $ss DivBox
MxQuietStatistics $DivBox { Mean Median } false 0.0001 Stats
set DivMean [ expr [keylget Stats Mean] / 100.0 ]
set DivMedian [ expr [keylget Stats Median] / 100.0 ]

#}}}

MxSetCurrentPage $OrigGroup

puts "Mean (StdDev Image / Mean Image)=$DivMean %\nMedian (StdDev Image / Mean Image)=$DivMedian %"

MxPause "Mean (StdDev Image/\nMean Image)=$DivMean%\n\nMedian (StdDev Image/\nMean Image)=$DivMedian%"

MxDeleteGroupPages { $Group } No
